package com.hzya.frame.plugin.lets.dingtalk.push.serivce.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenResponseBody;
import com.aliyun.dingtalkworkflow_1_0.models.StartProcessInstanceRequest;
import com.aliyun.dingtalkworkflow_1_0.models.StartProcessInstanceResponse;
import com.dingtalk.api.response.OapiV2DepartmentGetResponse;
import com.dingtalk.api.response.OapiV2UserGetResponse;
import com.dingtalk.api.response.OapiV2UserGetbymobileResponse;
import com.hzya.frame.plugin.lets.dingtalk.dao.IDingU8cBillDao;
import com.hzya.frame.plugin.lets.dingtalk.entity.DingU8cBillEntity;
import com.hzya.frame.plugin.lets.dingtalk.push.serivce.PushU8CService;
import com.hzya.frame.plugin.lets.dingtalk.utils.DingTalkUtils;
import com.hzya.frame.plugin.lets.dingtalk.utils.FileUtil;
import com.hzya.frame.plugin.lets.dingtalk.vo.*;
import com.hzya.frame.plugin.lets.plugin.adjust.AdjustInPluginInitializer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Base64;
import java.util.Date;
import java.util.List;

/**
 * u8c单据推送钉钉表单
 * fileBase64s为文件转了base64之后
 */
@Service("PushU8CService")
public class PushU8CServiceImpl implements PushU8CService {

    @Value("${DING.U8C_TEST_AGENTID}")
    private String U8C_TEST_AGENTID;
    @Value("${DING.CorpId}")
    private String CorpId;
    @Value("${DING.APIToken}")
    private String APIToken;

    @Value("${DING.APPKEY}")
    private String APPKEY;
    @Value("${DING.AppSecret}")
    private String AppSecret;
    @Value("${DING.SPACEID}")
    private String SPACEID;

    @Value("${DING.APPROVE_CODE.SPECIAL_SERVICE_PROCESSING}")
    private String SPECIAL_SERVICE_PROCESSING;
    @Value("${DING.APPROVE_FOLDER_SPACE.SPECIAL_SERVICE_PROCESSING}")
    private String SPECIAL_SERVICE_PROCESSING_FOLDER;

    @Value("${DING.APPROVE_CODE.PURCHASE_PAYMENT_REQUEST_NEW}")
    private String PURCHASE_PAYMENT_REQUEST_NEW;
    @Value("${DING.APPROVE_FOLDER_SPACE.PURCHASE_PAYMENT_REQUEST_NEW}")
    private String PURCHASE_PAYMENT_REQUEST_NEW_FOLDER;

    @Value("${DING.APPROVE_CODE.PURCHASE_PAYMENT_REQUEST_SHOP}")
    private String PURCHASE_PAYMENT_REQUEST_SHOP;
    @Value("${DING.APPROVE_FOLDER_SPACE.PURCHASE_PAYMENT_REQUEST_SHOP}")
    private String PURCHASE_PAYMENT_REQUEST_SHOP_FOLDER;

    Logger logger = LoggerFactory.getLogger(PushU8CServiceImpl.class);

    @Autowired
    private IDingU8cBillDao dingU8cBillDao;


    /**
     * 委外采购（付款单）->采购付款申请（新）
     */
    @Override
    public String outSource(DjFileVO djFileVO) {
        String instanceId = "";
        //校验参数
        try {
            checkParameter(djFileVO);
            //获取钉钉参数
            GetAccessTokenResponseBody tokenBody = DingTalkUtils.getTokenBody(APPKEY, AppSecret);

            //制单人的钉钉unionid：通过手机号获取钉钉的userid在获取unionid   13783530043
            if (djFileVO.getDjzbVO().getParentvo().getLrr_code() == null || "".equals(djFileVO.getDjzbVO().getParentvo().getLrr_code())) {
                Assert.state(false, "录入人手机号为空！！！");
            }
            OapiV2UserGetbymobileResponse userByMobile = DingTalkUtils.getUserByMobile(tokenBody.getAccessToken(), djFileVO.getDjzbVO().getParentvo().getLrr_code());
            if(userByMobile.getResult()==null){
                Assert.state(false,userByMobile.getErrmsg());
            }
            String userid = userByMobile.getResult().getUserid();
            OapiV2UserGetResponse userById = DingTalkUtils.getUserById(tokenBody.getAccessToken(), userid);
            String unionid = userById.getResult().getUnionid();

            //base转文件流-上传钉盘-拿到钉盘信息。可能是List集合
            List<FileModuleVO> fileModuleVOList = baseTransformFile2uploadDing(djFileVO.getSysFileVOS(), tokenBody.getAccessToken(), unionid, SPACEID, PURCHASE_PAYMENT_REQUEST_NEW_FOLDER);

            //推送钉钉--》发起审批实例
            instanceId = excuteDingTalkOutSource(tokenBody.getAccessToken(), userid, PURCHASE_PAYMENT_REQUEST_NEW, djFileVO.getDjzbVO(), fileModuleVOList);

        } catch (Exception e) {
            logger.error("委外采购（付款单）->采购付款申请（新）报错", e);
            DingU8cBillEntity dingU8cBillEntity = new DingU8cBillEntity(djFileVO.getDjzbVO().getParentvo().getDwbm(), djFileVO.getDjzbVO().getParentvo().getDwbm_code(), djFileVO.getDjzbVO().getParentvo().getBusitypecode(), djFileVO.getDjzbVO().getParentvo().getDjbh(), djFileVO.getDjzbVO().getParentvo().getVouchid(), "N", e.getMessage(), null,null);
            dingU8cBillEntity.setCreate_time(new Date());
            dingU8cBillEntity.setU8c_name_corp(djFileVO.getDjzbVO().getParentvo().getDwbm_name());
            dingU8cBillDao.pushSaveOrUpdate(dingU8cBillEntity);
            Assert.state(false,e.getMessage());
        }
        return instanceId;
    }

    /**
     * 代理采购（付款单）->采购付款申请（店群专用）
     */
    @Override
    public String agency(DjFileVO djFileVO) {
        String instanceId = "";
        try {
            checkParameter(djFileVO);
            //获取钉钉参数
            GetAccessTokenResponseBody tokenBody = DingTalkUtils.getTokenBody(APPKEY, AppSecret);
            //制单人的钉钉unionid：通过手机号获取钉钉的userid在获取unionid   13783530043
            if (djFileVO.getDjzbVO().getParentvo().getLrr_code() == null || "".equals(djFileVO.getDjzbVO().getParentvo().getLrr_code())) {
                Assert.state(false, "录入人手机号为空！！！");
            }
            //制单人的钉钉unionid：通过手机号获取钉钉的userid在获取unionid   13783530043
            OapiV2UserGetbymobileResponse userByMobile = DingTalkUtils.getUserByMobile(tokenBody.getAccessToken(), djFileVO.getDjzbVO().getParentvo().getLrr_code());
            if(userByMobile.getResult()==null){
                Assert.state(false,userByMobile.getErrmsg());
            }
            String userid = userByMobile.getResult().getUserid();
            OapiV2UserGetResponse userById = DingTalkUtils.getUserById(tokenBody.getAccessToken(), userid);
            String unionid = userById.getResult().getUnionid();

            //base转文件流-上传钉盘-拿到钉盘信息。可能是List集合
            List<FileModuleVO> fileModuleVOList = baseTransformFile2uploadDing(djFileVO.getSysFileVOS(), tokenBody.getAccessToken(), unionid, SPACEID, PURCHASE_PAYMENT_REQUEST_SHOP_FOLDER);

            //推送钉钉--》发起审批实例
            instanceId = excuteDingTalkAgency(tokenBody.getAccessToken(), userid, PURCHASE_PAYMENT_REQUEST_SHOP, djFileVO.getDjzbVO(), fileModuleVOList);


        } catch (Exception e) {
            logger.error("代理采购（付款单）->采购付款申请（店群专用）报错", e);
            DingU8cBillEntity dingU8cBillEntity = new DingU8cBillEntity(djFileVO.getDjzbVO().getParentvo().getDwbm(), djFileVO.getDjzbVO().getParentvo().getDwbm_code(), djFileVO.getDjzbVO().getParentvo().getBusitypecode(), djFileVO.getDjzbVO().getParentvo().getDjbh(), djFileVO.getDjzbVO().getParentvo().getVouchid(), "N", e.getMessage(), null,null);
            dingU8cBillEntity.setCreate_time(new Date());
            dingU8cBillEntity.setU8c_name_corp(djFileVO.getDjzbVO().getParentvo().getDwbm_name());
            dingU8cBillDao.pushSaveOrUpdate(dingU8cBillEntity);
            Assert.state(false,e.getMessage());
        }
        return instanceId;
    }

    /**
     * 调拨单内部交易（付款单）->特殊业务处理(新)
     */
    @Override
    public String transfer(DjFileVO djFileVO) {
        String instanceId = "";
        try {
            checkParameter(djFileVO);
            //获取钉钉参数
            GetAccessTokenResponseBody tokenBody = DingTalkUtils.getTokenBody(APPKEY, AppSecret);
            //制单人的钉钉unionid：通过手机号获取钉钉的userid在获取unionid   13783530043
            if (djFileVO.getDjzbVO().getParentvo().getLrr_code() == null || "".equals(djFileVO.getDjzbVO().getParentvo().getLrr_code())) {
                Assert.state(false, "录入人手机号为空！！！");
            }
            //制单人的钉钉unionid：通过手机号获取钉钉的userid在获取unionid   13783530043
            OapiV2UserGetbymobileResponse userByMobile = DingTalkUtils.getUserByMobile(tokenBody.getAccessToken(), djFileVO.getDjzbVO().getParentvo().getLrr_code());
            if(userByMobile.getResult()==null){
                Assert.state(false,userByMobile.getErrmsg());
            }
            String userid = userByMobile.getResult().getUserid();
            OapiV2UserGetResponse userById = DingTalkUtils.getUserById(tokenBody.getAccessToken(), userid);
            String unionid = userById.getResult().getUnionid();
            //base转文件流-上传钉盘-拿到钉盘信息。可能是List集合
            List<FileModuleVO> fileModuleVOList = baseTransformFile2uploadDing(djFileVO.getSysFileVOS(), tokenBody.getAccessToken(), unionid, SPACEID, SPECIAL_SERVICE_PROCESSING_FOLDER);

            //推送钉钉--》发起审批实例
            instanceId = excuteDingTalkTransfer(tokenBody.getAccessToken(), userid, SPECIAL_SERVICE_PROCESSING, djFileVO.getDjzbVO(), fileModuleVOList);

        } catch (Exception e) {
            logger.error("调拨单内部交易（付款单）->特殊业务处理(新)报错", e);
            DingU8cBillEntity dingU8cBillEntity = new DingU8cBillEntity(djFileVO.getDjzbVO().getParentvo().getDwbm(), djFileVO.getDjzbVO().getParentvo().getDwbm_code(), djFileVO.getDjzbVO().getParentvo().getBusitypecode(), djFileVO.getDjzbVO().getParentvo().getDjbh(), djFileVO.getDjzbVO().getParentvo().getVouchid(), "N", e.getMessage(), null,null);
            dingU8cBillEntity.setCreate_time(new Date());
            dingU8cBillEntity.setU8c_name_corp(djFileVO.getDjzbVO().getParentvo().getDwbm_name());
            dingU8cBillDao.pushSaveOrUpdate(dingU8cBillEntity);
            Assert.state(false,e.getMessage());
        }
        return instanceId;
    }

    /**
     * 检查参数，VO不能为空，文件不能空
     *
     * @param djFileVO:djzbVO+sysFileVOS
     */
    public void checkParameter(DjFileVO djFileVO) {
        Assert.notNull(djFileVO.getDjzbVO(), "u8c单据推送钉钉表单,检查参数，VO不能为空");
        if (djFileVO.getSysFileVOS().size() == 0) {
            Assert.state(false, "u8c单据推送钉钉表单,检查参数，文件不能空");
        }
    }

    /**
     * 文件转换
     * 将base64转为文件流上传钉盘，并返回钉钉的 List<FileModuleVO>
     *
     * @Param fileVOS 文件集合
     * @Param token token
     * @Param unionid unionid
     * @Param spaceId 空间信息
     * @Param dingFileUrlID 钉盘审批实例中附件地址
     */
    public List<FileModuleVO> baseTransformFile2uploadDing(List<SysFileVO> fileVOS, String token, String unionid, String spaceId, String dingFileUrlID) throws IOException {
        List<FileModuleVO> fileModuleVOList = new ArrayList<>();
        for (SysFileVO fileVO : fileVOS) {
            // 指定输出文件路径
            Path outputPath = Paths.get("D:\\ding-file\\"+fileVO.getFileName());
            // 1. 解码Base64字符串并写入文件
            byte[] decodedBytes = Base64.getDecoder().decode(fileVO.getBaseCode());
            Files.write(outputPath, decodedBytes);
            // 获取文件大小
            String fileSizeInBytes = String.valueOf(Files.size(outputPath));
            fileVO.setFileSize(fileSizeInBytes);
            logger.info("文件已成功创建: " + outputPath);
            // 2. 将文件内容读取到ByteArrayOutputStream
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            try (InputStream inputStream = Files.newInputStream(outputPath)) {
                byte[] buffer = new byte[1024];
                int bytesRead;
                while ((bytesRead = inputStream.read(buffer)) != -1) {
                    byteArrayOutputStream.write(buffer, 0, bytesRead);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            // 3. 打印ByteArrayOutputStream的内容（可选）
            byte[] fileContent = byteArrayOutputStream.toByteArray();
            // 4. 文件流上传钉盘
            FileModuleVO dingDileModuleVO = FileUtil.get4DingDileModuleVO(token, unionid, spaceId, fileVO, byteArrayOutputStream, dingFileUrlID);

            fileModuleVOList.add(dingDileModuleVO);
        }
        return fileModuleVOList;
    }




    /**
     * 委外采购（付款单）->采购付款申请(新)：推送钉钉审批实例
     *
     * @param accessToken      token
     * @param userId           userId
     * @param processCode      钉钉模板Code
     * @param djzbVO           U8C单据VO
     * @param fileModuleVOList 钉钉文件信息，用于上传审批实例使用
     */
    public String excuteDingTalkOutSource(String accessToken, String userId, String processCode, DjzbVO djzbVO, List<FileModuleVO> fileModuleVOList) throws Exception {

        DjzbVO.Arap_djzb parentvo = djzbVO.getParentvo();
        List<DjzbVO.Arap_djfb> childrens = djzbVO.getChildren();
        String instanceId = "";
        try {
            //部门查询
            Long dingDeptId = queryDingDeptId(parentvo.getDeptid_name(), accessToken, userId);
            if (dingDeptId == null) {
                Assert.state(false, "根据u8c部门查询钉钉部门失败，可能该录入人：{}，不在钉钉部门：{} 下。", parentvo.getLrr_name(), parentvo.getDeptid_name());
            }

            //formComponentValues-->表单数据内容，控件列表，最大列表长度：150。
            List<StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues> form = new ArrayList<>();

            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("费用归属公司").setValue(parentvo.getDwbm_name()));
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("费用归属部门（选择至末级部门）").setValue(parentvo.getDeptid_name()));
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("品牌").setValue(parentvo.getPinpai()));
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("费用所属类目").setValue(parentvo.getBusitypename()));
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("付款类型").setValue(parentvo.getPrepay()));
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("付款比例").setValue(parentvo.getFkbl()));
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("类目所属（供应链）").setValue("供应链类"));
//            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("项目编号").setValue("1"));
//            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("项目名称").setValue("1"));
//            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("类型").setValue("1"));
            //列表
            List<List<DingTableVO>> dingTableVOSMaterialList = new ArrayList<>();
            for (DjzbVO.Arap_djfb children : childrens) {
                List<DingTableVO> dingTableVOS = new ArrayList<>();
//                dingTableVOS.add(new DingTableVO("存货分类", "存货分类"));
                dingTableVOS.add(new DingTableVO("产品名称", children.getCinventoryid_name()));
                dingTableVOS.add(new DingTableVO("产品数量", children.getDfshl()));
                dingTableVOS.add(new DingTableVO("付款金额（元）", children.getBbye()));
                dingTableVOS.add(new DingTableVO("税率", children.getSl()));
                dingTableVOSMaterialList.add(dingTableVOS);
            }
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("付款明细").setValue(JSONUtil.toJsonStr(dingTableVOSMaterialList)));

//            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("开票抬头").setValue(parentvo.getDwname()));
//            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("开票方").setValue(parentvo.getCustname()));
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("对方类型").setValue("1"));
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("支付方式-银行").setValue(parentvo.getAccountname() + parentvo.getAccount() + parentvo.getBanktypename()));
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("银行支行名称").setValue(parentvo.getBankdocname()));
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("申请支付日期").setValue(parentvo.getDjrq()));
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("备注").setValue(parentvo.getScomment()));
            //附件
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setComponentType("DDAttachment").setId("DDAttachment_1PDRXW3QT4G00").setName("上传相关附件（业务相关证明、关联原合同审批件）").setValue(JSONUtil.toJsonStr(fileModuleVOList)));
            /**
             * 推送钉钉--》发起审批实例
             * accessToken-->
             * originatorUserId-->审批发起人的userId。
             * processCode-->审批流的唯一码。
             * deptId-->审批发起人所在的部门ID。
             * microappAgentId-->应用标识AgentId，
             * approvers-->不使用审批流模板时，直接指定的审批人列表，最大列表长度：20。
             * formComponentValues-->表单数据内容，控件列表，最大列表长度：150。
             */
            StartProcessInstanceResponse startProcessInstanceResponse = DingTalkUtils.InitiateApprovalInstance(accessToken, userId, processCode, dingDeptId, Long.valueOf(U8C_TEST_AGENTID), null, form);
            JSONObject jsonObject = JSONUtil.parseObj(startProcessInstanceResponse);
            String statusCode = jsonObject.get("statusCode").toString();
            if (statusCode != null && !statusCode.equals("200")) {
                Assert.state(false, "委外采购（付款单）->采购付款申请(新)：推送钉钉审批实例失败。失败原因：{}", JSONUtil.toJsonStr(jsonObject.get("body")));
            }
            logger.info("推送钉钉-->发起审批实例：委外采购（付款单）->采购付款申请(新)-->执行成功");

            //审批实例id
            instanceId = JSONUtil.parseObj(jsonObject.get("body")).get("instanceId").toString();

            //记录成功记录
            DingU8cBillEntity dingU8cBillEntity = new DingU8cBillEntity(parentvo.getDwbm(), parentvo.getDwbm_code(), parentvo.getBusitypecode(), parentvo.getDjbh(), parentvo.getVouchid(), "Y", "success", instanceId,null);
//            DingU8cBillEntity dingU8cBillEntity = new DingU8cBillEntity(parentvo.getDwbm(), parentvo.getDwbm_code(), parentvo.getBusitypecode(), parentvo.getDjbh(), parentvo.getVouchid(), null, null, null,null);
            dingU8cBillEntity.setCreate_time(new Date());
            dingU8cBillEntity.setU8c_name_corp(parentvo.getDwbm_name());
            dingU8cBillDao.pushSaveOrUpdate(dingU8cBillEntity);
            return instanceId;

        } catch (Exception e) {
            logger.error(e.getMessage());
            //记录失败记录
//            DingU8cBillEntity dingU8cBillEntity = new DingU8cBillEntity(parentvo.getDwbm(), parentvo.getDwbm_code(), parentvo.getBusitypecode(), parentvo.getDjbh(), parentvo.getVouchid(), "N", e.getMessage(), null,null);
//            dingU8cBillEntity.setCreate_time(new Date());
//            dingU8cBillDao.pushSaveOrUpdate(dingU8cBillEntity);
            Assert.state(false,"推送钉钉失败，失败原因：{}",e.getMessage());
        }
        return null;
    }


    /**
     * 代理采购（付款单）->采购付款申请（店群专用）
     */
    public String excuteDingTalkAgency(String accessToken, String userId, String processCode, DjzbVO djzbVO, List<FileModuleVO> fileModuleVOList) throws Exception {
        DjzbVO.Arap_djzb parentvo = djzbVO.getParentvo();
        List<DjzbVO.Arap_djfb> childrens = djzbVO.getChildren();
        String instanceId = "";
        try {
            //部门查询
            Long dingDeptId = queryDingDeptId(parentvo.getDeptid_name(), accessToken, userId);
            if (dingDeptId == null) {
                Assert.state(false, "根据u8c部门查询钉钉部门失败，可能该录入人：{}，不在钉钉部门：{} 下。", parentvo.getLrr_name(), parentvo.getDeptid_name());
            }
            //formComponentValues-->表单数据内容，控件列表，最大列表长度：150。
            List<StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues> form = new ArrayList<>();
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("费用归属公司").setValue(parentvo.getDwbm_name()));
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("费用归属部门（选择至末级部门）").setValue(parentvo.getDeptid_name()));
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("项目所属类目").setValue("红人代运营"));//固定值
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("费用所属类目").setValue("运营类"));//固定值
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("总价").setValue(parentvo.getNtaxpricemny()));
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("实际应支付金额").setValue(parentvo.getBbje()));
            if(parentvo.getGdbh()!=null&&"".equals(parentvo.getGdbh())){
                form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("归档编号").setValue(parentvo.getGdbh()));
            }
//            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("备注").setValue(parentvo.getScomment()));
            //附件
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setComponentType("DDAttachment").setId("DDAttachment_D7JK56572Y00").setName("附件").setValue(JSONUtil.toJsonStr(fileModuleVOList)));

            //推送钉钉审批实例
            StartProcessInstanceResponse startProcessInstanceResponse = DingTalkUtils.InitiateApprovalInstance(accessToken, userId, processCode, dingDeptId, Long.valueOf(U8C_TEST_AGENTID), null, form);
            JSONObject jsonObject = JSONUtil.parseObj(startProcessInstanceResponse);
            String statusCode = jsonObject.get("statusCode").toString();
            if (statusCode != null && !statusCode.equals("200")) {
                Assert.state(false, "代理采购（付款单）->采购付款申请（店群专用）：推送钉钉审批实例失败。失败原因：{}", JSONUtil.toJsonStr(jsonObject.get("body")));
            }
            logger.info("推送钉钉-->发起审批实例：代理采购（付款单）->采购付款申请（店群专用）-->执行成功");

            instanceId = JSONUtil.parseObj(jsonObject.get("body")).get("instanceId").toString();

            //记录成功记录
            DingU8cBillEntity dingU8cBillEntity = new DingU8cBillEntity(parentvo.getDwbm(), parentvo.getDwbm_code(), parentvo.getBusitypecode(), parentvo.getDjbh(), parentvo.getVouchid(), "Y", "success", instanceId,null);
            dingU8cBillEntity.setCreate_time(new Date());
            dingU8cBillEntity.setU8c_name_corp(parentvo.getDwbm_name());
            dingU8cBillDao.pushSaveOrUpdate(dingU8cBillEntity);
            return instanceId;

        } catch (Exception e) {
            logger.error(e.getMessage());
            //记录失败记录
//            DingU8cBillEntity dingU8cBillEntity = new DingU8cBillEntity(parentvo.getDwbm(), parentvo.getDwbm_code(), parentvo.getBusitypecode(), parentvo.getDjbh(), parentvo.getVouchid(), "N", e.getMessage(), null,null);
//            dingU8cBillEntity.setCreate_time(new Date());
//            dingU8cBillDao.pushSaveOrUpdate(dingU8cBillEntity);
            Assert.state(false,"推送钉钉失败，失败原因：{}",e.getMessage());
        }
        return instanceId;
    }

    /**
     * 调拨单内部交易（付款单）->特殊业务处理(新)
     */
    public String excuteDingTalkTransfer(String accessToken, String userId, String processCode, DjzbVO djzbVO, List<FileModuleVO> fileModuleVOList) throws Exception {
        DjzbVO.Arap_djzb parentvo = djzbVO.getParentvo();
        List<DjzbVO.Arap_djfb> childrens = djzbVO.getChildren();
        String instanceId = "";
        try {
            //部门查询
            Long dingDeptId = queryDingDeptId(parentvo.getDeptid_name(), accessToken, userId);
            if (dingDeptId == null) {
                Assert.state(false, "根据u8c部门查询钉钉部门失败，可能该录入人：{}，不在钉钉部门：{} 下。", parentvo.getLrr_name(), parentvo.getDeptid_name());
            }
            //formComponentValues-->表单数据内容，控件列表，最大列表长度：150。
            List<StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues> form = new ArrayList<>();
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("费用所属公司").setValue(parentvo.getDwbm_name()));
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("费用所属部门").setValue(parentvo.getDeptid_name()));
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("项目所属类目").setValue("综合"));//固定值
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("费用所属项目").setValue("内部往来"));//固定值
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("申请事由").setValue(parentvo.getScomment()));
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("申请金额（元）").setValue(parentvo.getBbje()));
//            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setName("备注").setValue(parentvo.getScomment()));
            //附件
            form.add(new StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues().setComponentType("DDAttachment").setId("DDAttachment_1XN8VWYAKBTS0").setName("附件").setValue(JSONUtil.toJsonStr(fileModuleVOList)));
            //推送钉钉审批实例
            StartProcessInstanceResponse startProcessInstanceResponse = DingTalkUtils.InitiateApprovalInstance(accessToken, userId, processCode, dingDeptId, Long.valueOf(U8C_TEST_AGENTID), null, form);
            JSONObject jsonObject = JSONUtil.parseObj(startProcessInstanceResponse);
            String statusCode = jsonObject.get("statusCode").toString();
            if (statusCode != null && !statusCode.equals("200")) {
                Assert.state(false, "调拨单内部交易（付款单）->特殊业务处理(新)：推送钉钉审批实例失败。失败原因：{}", JSONUtil.toJsonStr(jsonObject.get("body")));
            }
            logger.info("推送钉钉-->发起审批实例：调拨单内部交易（付款单）->特殊业务处理(新)-->执行成功");

            instanceId = JSONUtil.parseObj(jsonObject.get("body")).get("instanceId").toString();

            //记录成功记录
            DingU8cBillEntity dingU8cBillEntity = new DingU8cBillEntity(parentvo.getDwbm(), parentvo.getDwbm_code(), parentvo.getBusitypecode(), parentvo.getDjbh(), parentvo.getVouchid(), "Y", "success", instanceId,null);
            dingU8cBillEntity.setCreate_time(new Date());
            dingU8cBillEntity.setU8c_name_corp(parentvo.getDwbm_name());
            dingU8cBillDao.pushSaveOrUpdate(dingU8cBillEntity);

        } catch (Exception e) {
            logger.error(e.getMessage());
            //记录失败记录
//            DingU8cBillEntity dingU8cBillEntity = new DingU8cBillEntity(parentvo.getDwbm(), parentvo.getDwbm_code(), parentvo.getBusitypecode(), parentvo.getDjbh(), parentvo.getVouchid(), "N", e.getMessage(), null,null);
//            dingU8cBillEntity.setCreate_time(new Date());
//            dingU8cBillDao.pushSaveOrUpdate(dingU8cBillEntity);
            Assert.state(false,"推送钉钉失败，失败原因：{}",e.getMessage());
        }

        return instanceId;
    }


    /**
     * 查询钉钉部门
     *
     * @return 钉钉部门id
     * @Param u8cDeptName u8c部门名称
     * @Param token
     * @Param userId
     */
    public Long queryDingDeptId(String u8cDeptName, String token, String userId) {
        try {
            Assert.notNull(u8cDeptName, "查询钉钉部门u8c部门名称不能为空！！！");
            OapiV2UserGetResponse user = DingTalkUtils.getUserById(token, userId);
            List<Long> deptIdList = user.getResult().getDeptIdList();
            //查询钉钉部门
            for (Long deptId : deptIdList) {
                OapiV2DepartmentGetResponse dingDept = DingTalkUtils.getDeptByDeptId(token, deptId);
                String dingDeptName = dingDept.getResult().getName();
                if (u8cDeptName.equals(dingDeptName)) {
                    return deptId;
                }
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
        }
        return null;
    }
}
